"""
Clase híbrida para conversión LaTeX - ADAPTADA AL NUEVO SISTEMA
"""
from sympy_Base import SympyClassBase
import sympy
import re


class Class_LaTeX(SympyClassBase):
    """Clase híbrida para conversión de expresiones a formato LaTeX y viceversa"""
    
    def __new__(cls, value_input):
        """Crear objeto SymPy válido"""
        obj = SympyClassBase.__new__(cls)
        return obj
    
    def __init__(self, expression):
        """Inicialización de LaTeX"""
        # Si el input es string, intentar parsearlo como expresión de SymPy
        if isinstance(expression, str):
            try:
                # Parsear la expresión string a objeto SymPy
                parsed_expr = sympy.sympify(expression)
                self._expression = parsed_expr
            except Exception as e:
                # Si no se puede parsear, mantener como string
                self._expression = expression
        else:
            # Si ya es un objeto SymPy, usarlo directamente
            self._expression = expression
        
        # Generar código LaTeX
        try:
            if hasattr(self._expression, '__class__') and hasattr(sympy, 'latex'):
                self._latex_code = sympy.latex(self._expression)
            else:
                self._latex_code = str(self._expression)
        except Exception:
            self._latex_code = str(self._expression)
        
        # Llamar al constructor base con el código LaTeX como valor de display
        super().__init__(self._expression, self._latex_code)
    
    def __str__(self):
        """Representación string que muestra el código LaTeX"""
        return self._latex_code
    
    def _sympystr(self, printer):
        """Representación SymPy string"""
        return self._latex_code
    
    def latex_code(self):
        """Retorna el código LaTeX generado"""
        return self._latex_code
    
    def original_expression(self):
        """Retorna la expresión original"""
        return self._expression
    
    @staticmethod
    def parse_latex(latex_string):
        """
        Convierte código LaTeX a expresión SymPy usando parse_latex de SymPy
        
        Args:
            latex_string (str): Código LaTeX a convertir
            
        Returns:
            Class_LaTeX: Nueva instancia con la expresión parseada
            
        Examples:
            LaTeX.parse_latex(r"$$Brix_{Bev} = \frac{Brix_{syr} + Brix_{H_2O} \cdot R_M}{R_M + 1}$$")
        """
        # Primero intentar conversión manual mejorada (más confiable para subíndices)
        try:
            manual_expr = Class_LaTeX._manual_latex_to_sympy(latex_string)
            return Class_LaTeX(manual_expr)
        except Exception as e1:
            # Si falla la conversión manual, intentar parse_latex de SymPy
            try:
                # Importar parse_latex de SymPy
                from sympy.parsing.latex import parse_latex
                
                # Limpiar el string LaTeX (eliminar $$ si existen)
                clean_latex = latex_string.strip()
                if clean_latex.startswith('$$') and clean_latex.endswith('$$'):
                    clean_latex = clean_latex[2:-2].strip()
                elif clean_latex.startswith('$') and clean_latex.endswith('$'):
                    clean_latex = clean_latex[1:-1].strip()
                
                # Intentar parsing directo con SymPy (solo para LaTeX simple)
                parsed_expr = parse_latex(clean_latex)
                
                # Crear nueva instancia de Class_LaTeX
                return Class_LaTeX(parsed_expr)
                
            except Exception as e2:
                print(f"Error parseando LaTeX '{latex_string}': {e1}")
                print(f"Error con parse_latex de SymPy: {e2}")
                # Retornar una instancia con un símbolo que represente la expresión LaTeX
                # Usar el nombre más simple posible
                simplified_name = re.sub(r'[^a-zA-Z0-9_]', '_', latex_string.replace('$$', '').replace('$', ''))[:20]
                fallback_expr = sympy.Symbol(simplified_name or 'LaTeX_expr')
                result = Class_LaTeX(fallback_expr)
                result._latex_code = latex_string  # Mantener el LaTeX original
                return result
    
    @staticmethod
    def _preprocess_subscripts(latex_str):
        """
        Preprocesa subíndices largos para que parse_latex los maneje mejor
        Estrategia: Mapear subíndices complejos a nombres simples temporalmente
        """
        # Diccionario para mapear subíndices complejos a nombres simples
        subscript_mapping = {}
        simple_counter = 1
        
        # Patrón para encontrar subíndices con llaves: var_{contenido}
        pattern = r'([a-zA-Z][a-zA-Z0-9]*)\_{([^}]+)}'
        
        def replace_subscript(match):
            nonlocal simple_counter
            var_name = match.group(1)
            subscript = match.group(2)
            
            # Crear un nombre simplificado pero único
            original_full_name = f"{var_name}_{subscript}"
            
            # Limpiar el subíndice de caracteres problemáticos
            clean_subscript = re.sub(r'[^a-zA-Z0-9]', '', subscript)
            
            # Si el subíndice limpio es simple, usarlo directamente
            if len(clean_subscript) <= 5 and clean_subscript.isalnum():
                simple_name = f"{var_name}{clean_subscript}"
            else:
                # Para subíndices complejos, usar un identificador simple
                simple_name = f"{var_name}sub{simple_counter}"
                simple_counter += 1
            
            # Guardar el mapeo para restaurar después
            subscript_mapping[simple_name] = original_full_name.replace('_', '__')
            
            return simple_name
        
        processed_str = re.sub(pattern, replace_subscript, latex_str)
        
        # Guardar el mapeo en el string para recuperarlo después
        # (Hack: usar un comentario que no afecte el parsing)
        if subscript_mapping:
            mapping_str = "|".join([f"{k}:{v}" for k, v in subscript_mapping.items()])
            processed_str = f"{processed_str} # SUBSCRIPT_MAP:{mapping_str}"
        
        return processed_str
    
    @staticmethod
    def _postprocess_subscripts(expr, mapping_info=None):
        """
        Post-procesa la expresión para restaurar los nombres originales de subíndices
        """
        if hasattr(expr, 'free_symbols'):
            substitutions = {}
            
            # Si tenemos información de mapeo, usarla para restaurar nombres
            if mapping_info:
                for symbol in expr.free_symbols:
                    symbol_name = str(symbol)
                    if symbol_name in mapping_info:
                        original_name = mapping_info[symbol_name]
                        new_symbol = sympy.Symbol(original_name)
                        substitutions[symbol] = new_symbol
            
            # Para símbolos que no están en el mapeo, aplicar reglas estándar
            for symbol in expr.free_symbols:
                symbol_name = str(symbol)
                if symbol not in substitutions:
                    # Mantener símbolos simples como están
                    if not any(char in symbol_name for char in ['_', '__']):
                        continue
                    # Crear nuevo símbolo con el nombre apropiado
                    new_symbol = sympy.Symbol(symbol_name)
                    if symbol != new_symbol:
                        substitutions[symbol] = new_symbol
            
            # Aplicar sustituciones si las hay
            if substitutions:
                expr = expr.subs(substitutions)
        
        return expr
    
    @staticmethod
    def _manual_latex_to_sympy(latex_str):
        """
        Conversión manual mejorada de LaTeX a SymPy como respaldo
        Usa un enfoque step-by-step para manejar subíndices complejos
        """
        # Limpiar el string
        clean_str = latex_str.strip()
        if clean_str.startswith('$$') and clean_str.endswith('$$'):
            clean_str = clean_str[2:-2].strip()
        elif clean_str.startswith('$') and clean_str.endswith('$'):
            clean_str = clean_str[1:-1].strip()
        
        # Determinar si es una ecuación
        is_equation = '=' in clean_str and not any(op in clean_str for op in ['==', '!=', '<=', '>='])
        
        # PASO 1: Identificar y mapear variables con subíndices ANTES de hacer otras conversiones
        variable_mapping = {}
        counter = 1
        
        # Patrón para variables con subíndices: Var_{subscript}
        def replace_subscripted_var(match):
            nonlocal counter
            full_match = match.group(0)
            var_base = match.group(1)
            subscript = match.group(2)
            
            # Crear un nombre temporal único
            temp_name = f"VAR{counter}"
            counter += 1
            
            # Crear el nombre final con doble guión bajo
            final_name = f"{var_base}__{subscript.replace(' ', '').replace('_', '')}"
            variable_mapping[temp_name] = final_name
            
            return temp_name
        
        # Aplicar el mapeo de variables
        result = re.sub(r'([a-zA-Z][a-zA-Z0-9]*?)_\{([^}]+)\}', replace_subscripted_var, clean_str)
        
        # PASO 2: Conversiones de operadores LaTeX  
        conversions = [
            (r'\\frac\{([^}]*)\}\{([^}]*)\}', r'(\1)/(\2)'),  # fracciones
            (r'\\sqrt\{([^}]*)\}', r'sqrt(\1)'),  # raíces cuadradas
            (r'\\cdot', '*'),  # multiplicación
            (r'\\times', '*'),  # multiplicación  
            (r'\\div', '/'),  # división
            (r'\{([^}]*)\}', r'\1'),  # eliminar llaves restantes
        ]
        
        for pattern, replacement in conversions:
            result = re.sub(pattern, replacement, result)
        
        # PASO 3: Limpiar espacios extra
        result = re.sub(r'\s+', ' ', result).strip()
        
        # PASO 4: Restaurar nombres de variables reales
        for temp_name, real_name in variable_mapping.items():
            result = result.replace(temp_name, real_name)
        
        # PASO 5: Intentar parsear como expresión SymPy
        try:
            if is_equation:
                # Para ecuaciones, usar Eq
                left, right = result.split('=', 1)
                left_expr = sympy.sympify(left.strip())
                right_expr = sympy.sympify(right.strip())
                return sympy.Eq(left_expr, right_expr)
            else:
                return sympy.sympify(result)
        except Exception as e:
            # PASO 6: Si falla sympify, crear símbolos explícitamente
            print(f"Debug: Resultado intermedio: '{result}'")
            print(f"Debug: Mapeo de variables: {variable_mapping}")
            
            # Crear los símbolos explícitamente
            symbol_names = set()
            
            # Extraer nombres de variables del resultado
            var_matches = re.findall(r'[a-zA-Z_][a-zA-Z0-9_]*', result)
            for var_name in var_matches:
                symbol_names.add(var_name)
            
            # Crear los símbolos
            symbols_dict = {}
            for name in symbol_names:
                symbols_dict[name] = sympy.Symbol(name)
            
            if symbols_dict:
                # Si es una ecuación, intentar crearla manualmente
                if is_equation and '=' in result:
                    left, right = result.split('=', 1)
                    # Crear una ecuación simple usando el primer símbolo encontrado
                    first_symbol = list(symbols_dict.values())[0]
                    return sympy.Eq(first_symbol, sum(list(symbols_dict.values())[1:], 0))
                else:
                    # Para expresiones, devolver el primer símbolo o una suma
                    symbols_list = list(symbols_dict.values())
                    if len(symbols_list) == 1:
                        return symbols_list[0]
                    else:
                        return sum(symbols_list[1:], symbols_list[0])
            else:
                return sympy.Symbol('LaTeX_parse_error')
    
    def to_image(self, filename=None, dpi=150):
        """
        Convierte el LaTeX a imagen (requiere matplotlib)
        """
        try:
            import matplotlib.pyplot as plt
            import matplotlib
            
            # Configurar matplotlib para usar LaTeX
            matplotlib.rcParams['text.usetex'] = True
            
            fig, ax = plt.subplots(figsize=(10, 2))
            ax.text(0.5, 0.5, f'${self._latex_code}$', 
                   horizontalalignment='center',
                   verticalalignment='center',
                   fontsize=16,
                   transform=ax.transAxes)
            ax.axis('off')
            
            if filename:
                plt.savefig(filename, dpi=dpi, bbox_inches='tight')
                plt.close()
                return f"LaTeX guardado como imagen en: {filename}"
            else:
                plt.show()
                return "LaTeX mostrado en pantalla"
                
        except ImportError:
            return "Error: matplotlib no está instalado. Instalar con: pip install matplotlib"
        except Exception as e:
            return f"Error generando imagen: {e}"

    @staticmethod
    def Helper(input_str):
        """Ayuda contextual para LaTeX"""
        if re.match(r"^\s*[Ll]atex\b", input_str, re.IGNORECASE):
            return r'''LaTeX - Conversión bidireccional de expresiones matemáticas y código LaTeX

Uso: 
  LaTeX[expresion]              # Convierte expresión a LaTeX
  LaTeX.parse_latex("latex")    # Convierte LaTeX a expresión
  
Ejemplos de conversión a LaTeX:
  LaTeX[x**2 + 2*x + 1]    → x^{2} + 2 x + 1
  LaTeX[sin(x)/cos(x)]     → \frac{\sin{\left(x \right)}}{\cos{\left(x \right)}}
  LaTeX[Integral(x, x)]    → \int x\,dx
  LaTeX[sqrt(x**2 + y**2)] → \sqrt{x^{2} + y^{2}}

Ejemplos de parsing desde LaTeX:
  LaTeX.parse_latex("$$Brix_{Bev} = \frac{Brix_{syr} + Brix_{H_2O} \cdot R_M}{R_M + 1}$$")
  LaTeX.parse_latex("\frac{x^2 + 1}{x - 1}")
  LaTeX.parse_latex("\sqrt{a^2 + b^2}")

Métodos disponibles:
  .latex_code()       - Obtiene el código LaTeX
  .original_expression() - Obtiene la expresión original
  .to_image()         - Convierte a imagen (requiere matplotlib)
  .parse_latex(str)   - Método estático para parsear LaTeX

Nota: Los subíndices se manejan con "_" (ej: Brix_Bev)
Las expresiones pueden ser strings o objetos SymPy'''
        return None

    @staticmethod
    def PopupFunctionList():
        """Lista de métodos sugeridos para autocompletado de LaTeX"""
        return [
            ("latex_code", "Obtiene el código LaTeX generado"),
            ("original_expression", "Obtiene la expresión matemática original"),
            ("to_image", "Convierte el LaTeX a imagen (requiere matplotlib)"),
            ("parse_latex", "Método estático para convertir LaTeX a expresión SymPy"),
        ]


def register_classes_in_module():
    """
    Devuelve una lista de clases definidas en este módulo para ser registradas.
    """
    return [
        ("LaTeX", Class_LaTeX, "SympyClassBase", {
            "add_lowercase": True,
            "supports_brackets": True,
            "description": "Conversión bidireccional entre expresiones matemáticas y formato LaTeX"
        }),
    ] 